@@ -37,83 +37,61 @@ class FocusedWave(AcousticField):
             return None
 
     def _apply_delay(self, dx=None):
-        """
-        Apply temporal delays to focus the wave at (self.focal_line, self.params['Foc']).
-        Only active elements around focal_line are considered (N_piezoFocal elements).
-        Returns:
-            ndarray: Delayed signals of shape (total_grid_points, len(burst) + max_delay).
-        """
         try:
             if dx is None:
                 dx = self.params['dx']
 
-            # 1. Calculate total grid points and points per element
+            # 1. Grid and element setup (utilise les paramètres utilisateur)
             element_width_grid_points = int(round(self.params['element_width'] / dx))
             total_grid_points = self.params['num_elements'] * element_width_grid_points
-
-            # 2. Calculate element center positions
             element_positions = np.linspace(
                 self.params['Xrange'][0] + self.params['element_width']/2,
-                self.params['Xrange'][1] - self.params['element_width']/2,
+                self.params['Xrange'][1] - self.params['element_width'],
                 self.params['num_elements']
             )
 
-            # 3. Determine active elements around focal_line
+            # 2. Select active elements (utilise les paramètres utilisateur)
             center_idx = np.argmin(np.abs(element_positions - self.focal_line))
-            start_idx = max(0, center_idx - self.params['N_piezoFocal'] // 2)
-            end_idx = min(self.params['num_elements'] - 1, start_idx + self.params['N_piezoFocal'] - 1)
+            half_N = self.params['N_piezoFocal'] // 2
+            start_idx = max(0, center_idx - half_N)
+            end_idx = min(self.params['num_elements'] - 1, center_idx + half_N - 1)
             active_elements = np.arange(start_idx, end_idx + 1)
-
-            # 4. Calculate all grid positions
-            grid_positions = np.linspace(
-                self.params['Xrange'][0],
-                self.params['Xrange'][1] - dx,
-                total_grid_points
-            )
-
-            # 5. Calculate maximum possible delay
-            # Calculer max_distance uniquement pour les éléments actifs
             active_element_positions = element_positions[active_elements]
-            max_distance = np.max(
-                np.sqrt(
-                    (active_element_positions - self.focal_line)**2 +
-                    self.params['Foc']**2
-                )
-            )
 
-            max_delay_samples = int(np.ceil(max_distance / self.params['c0'] / self.kgrid.dt))
+            # 3. Calculate relative positions
+            x_rel = active_element_positions - self.focal_line
+            a = np.max(np.abs(x_rel))
 
-            print(f"max delay (samples): {max_delay_samples}")
+            # 4. Calculate the parabolic delays (utilise les paramètres utilisateur)
+            tau_max = (a**2) / (2 * self.params['Foc'] * self.params['c0'])
+            delays = tau_max * (1 - (x_rel / a)**2)
 
-            # 6. Initialize output array
+            # 5. Force the delays at the edges to be zero
+            tau_edge = delays[0]
+            delays -= tau_edge
+
+            # 6. Convert delays to samples
+            delay_samples = np.round(delays / self.kgrid.dt).astype(int)
+            max_delay_samples = np.max(delay_samples)
+            print(f"Max delay (samples): {max_delay_samples}")
+
+            # 7. Initialize output
             delayed_signals = np.zeros((total_grid_points, len(self.burst) + max_delay_samples))
 
-            # 7. Calculate and apply delays for active elements only
-            for elem_idx in active_elements:
-                # Get grid indices for this element
+            # 8. Apply delays
+            for elem_idx, elem_delay in zip(active_elements, delay_samples):
                 start_grid = elem_idx * element_width_grid_points
                 end_grid = start_grid + element_width_grid_points
-
-                # Calculate delays for each grid point of this element
                 for grid_idx in range(start_grid, end_grid):
-                    # Distance to focal point (focal_line, Foc)
-                    distance = np.sqrt(
-                        (grid_positions[grid_idx] - self.focal_line)**2 +
-                        self.params['Foc']**2
-                    )
-
-                    # Time delay = (max_distance - distance) / speed_of_sound
-                    delay_samples = int(np.round((max_distance - distance) / self.params['c0'] / self.kgrid.dt))
-
-                    # Apply delay if valid
-                    if delay_samples >= 0 and delay_samples + len(self.burst) <= delayed_signals.shape[1]:
-                        delayed_signals[grid_idx, delay_samples:delay_samples + len(self.burst)] = self.burst
+                    if elem_delay >= 0 and elem_delay + len(self.burst) <= delayed_signals.shape[1]:
+                        delayed_signals[grid_idx, elem_delay:elem_delay + len(self.burst)] = self.burst
 
             return delayed_signals
 
         except Exception as e:
             print(f"Error applying delay: {e}")
             return None
+
  
     def plot_delay(self):
         """
